/*
 * @file       servo.c
 * @brief      manage 24 servo motors
 * 
 * @author     laurent Saint-Marcel(lstmarcel@yahoo.fr)
 * @date       2002/09/02
 */
#include <stdafx.h>

#include "uart.h"
#include "servo.h"

#define SERVO_SET_POS_CODE	0x10
#define SERVO_SET_MASK_CODE 0x00

static BYTE servoMask[SERVO_MASK_NUMBER];
BYTE servoPos[SERVO_NUMBER];

/* ------------------------------------------------------------------------*/
/* servoGetBoardInfo                                                       */
/* ------------------------------------------------------------------------*/
/* Private function that converts servoId to boardId/port/servoId          */
/* understandable by servo12.asm program                                   */
/* ------------------------------------------------------------------------*/
void servoGetBoardInfo(BYTE* servoId, 
					   BYTE* boardId, 
					   BYTE* port)
{
	*boardId=0;
	*port=0;
	if (*servoId>=12) {
		*boardId=1;
		*servoId-=12;
	}
	if (*servoId>=8) {
		*port=1;
		*servoId-=8;
	}
}

/* ------------------------------------------------------------------------*/
/* servoSetPos                                                             */
/* ------------------------------------------------------------------------*/
void servoSetPos(BYTE servoId, 
				 BYTE pos)
{
	BYTE boardId=0;
	BYTE port=0;
	servoPos[servoId]=pos;
	servoGetBoardInfo(&servoId, &boardId, &port);
	/* send header */
	uartSend((boardId<<5)+SERVO_SET_POS_CODE+(port<<3)+servoId);
	uartReceive(TRUE); /* wait ack */
	/* send servo position */
	uartSend(pos);
	uartReceive(TRUE); /* wait ack */
}

/* ------------------------------------------------------------------------*/
/* servoSetMask                                                            */
/* ------------------------------------------------------------------------*/
void servoSetMask(BYTE boardId, 
				  BYTE port, 
				  BYTE mask)
{
	/* store mask */
	servoMask[2*boardId+port]=mask;
	/* send header */
	uartSend((boardId<<5)+SERVO_SET_MASK_CODE+(port<<3));
	uartReceive(TRUE); /* wait ack */
	/* send servo position */
	uartSend(mask);
	uartReceive(TRUE); /* wait ack */
}

/* ------------------------------------------------------------------------*/
/* servoEnable                                                             */
/* ------------------------------------------------------------------------*/
void servoEnable(BYTE servoId, 
				 BOOL enable)
{
	BYTE boardId=0;
	BYTE port=0;
	BYTE maskId=0;
	servoGetBoardInfo(&servoId, &boardId, &port);
	maskId=2*boardId+port;
	if (enable)
		servoMask[maskId] |= 1<<servoId;
	else /* disable */
		servoMask[maskId] &= 0<<servoId;
	servoSetMask(boardId, port, servoMask[maskId]);
}

/* ------------------------------------------------------------------------*/
/* servoIsEnable                                                             */
/* ------------------------------------------------------------------------*/
BOOL servoIsEnable(BYTE servoId)
{
  BYTE boardId;
  BYTE portId;
  BYTE mask;
  servoGetBoardInfo(&boardId, &portId, &servoId);
  mask = 1<<servoId;
  return (servoMask[2*boardId+portId]&mask) == mask;
}

/* ------------------------------------------------------------------------*/
/* servoWait                                                             */
/* ------------------------------------------------------------------------*/
void servoWait(BYTE servoStep)
{
  sleep01(servoStep);
}
