/*
 * @file   spiderhead.c
 * @brief  Functions that control the robot head movements...
 *
 *
 * @author Laurent Saint-Marcel (lstmarcel@yahoo.fr)
 *
 * @date   2002/09/15
 */
#include <stdafx.h>

#include "spiderHead.h"
#include "servo.h"
#include "spiderIO.h"

#define HEAD_SERVO_SMILE   190
#define HEAD_SERVO_NOTHING 215
#define HEAD_SERVO_ANGRY   240

#define servoHeadId    3
#define servoLeapId   19

/*
 * @brief This function moves the robot forward in the mainLegId direction
 * @param dir  direction of the head : [HEAD_DIR_MIN..HEAD_DIR_MAX],
 *             can be HEAD_DIR_FRONT
 */
void headSetPosition(BYTE dir)
{
  if (dir>HEAD_DIR_MAX) dir=HEAD_DIR_MAX;
  else if (dir<HEAD_DIR_MIN) dir=HEAD_DIR_MIN;
  servoSetPos(servoHeadId, dir);
  if (!servoIsEnable(servoHeadId))
    servoEnable(servoHeadId, TRUE);
}

void headDisable()
{
  servoEnable(servoHeadId, FALSE);
  servoEnable(servoLeapId, FALSE);
}


/*
 * @brief The robot is happy !
 */
void headSmile()
{
  servoSetPos(servoLeapId, HEAD_SERVO_SMILE);
  if (!servoIsEnable(servoLeapId))
    servoEnable(servoLeapId, TRUE);
  ioSetLeapLeds(TRUE);
}


/*
 * @brief The robot is angry !
 */
void headAngry()
{
  servoSetPos(servoLeapId, HEAD_SERVO_ANGRY);
  if (!servoIsEnable(servoLeapId))
    servoEnable(servoLeapId, TRUE);
  ioSetLeapLeds(TRUE);
}


/*
 * @brief The robot face has no expression
 */
void headNothing()
{
  servoSetPos(servoLeapId, HEAD_SERVO_NOTHING);
  if (!servoIsEnable(servoLeapId))
    servoEnable(servoLeapId, TRUE);
  ioSetLeapLeds(FALSE);
}
