include <io.h>

#include "uart.h"

#ifdef UART_FULL
#include <string.h>
#endif



/* ------------------------------------------------------------------------*/
/* uartInit                                                                */
/* ------------------------------------------------------------------------*/
void uartInit(BYTE baud)
{
  /* set speed */
  outp(baud,UBRR);
  /* enable uart (2 directions: rx and tx) */
  outp((1 << RXEN) | (1 << TXEN),UCR);
}

/* ------------------------------------------------------------------------*/
/* uartSend                                                                */
/* ------------------------------------------------------------------------*/
void uartSend(BYTE data)
{
  /* wait for data register do be empty */
  while ( !(inp(USR) & (1<<UDRE)) );
  /* send data */
  outp(data,UDR);
}

/* ------------------------------------------------------------------------*/
/* uartReceive                                                             */
/* ------------------------------------------------------------------------*/
BYTE uartReceive(BOOL blocking)
{
  if (blocking) {
    /* wait for data register to be full */
    while ( !(inp(USR) & (1 << RXC)));
  } else if (!(inp(USR) & (1 << RXC)))
    /* data register empty, return the error code */
	return UART_NO_RCV;
  /* data register not empty, return the data */
  return inp(UDR);
}
  
/********************************************************************/
/*                      UART_FULL                                   */
/********************************************************************/

#ifdef UART_FULL
void uartSendString(char* string)
{
  int i;
  for (i=0 ; i < strlen(string) ; i++)
	{
	  uartSend(string[i]);
	}
}

void uartSendString_P(PGM_P string)
{
  int i;
  for (i=0 ; i < strlen_P(string) ; i++)
	{
	  uartSend(string[i]);
	}
}

void uartIntAffiche(int nb, int l)
{
  int tmp = nb;
  int i;
  if (tmp < 0)
    {
      uartSend('-');
      tmp = -tmp;
    }
  for (i = 0; i < l; i++)
    {
      uartSend(('0' +  tmp % 10));
      tmp = tmp / 10;
    }
}

void uartHexAffiche4(unsigned char data)
{
  unsigned char c = data & 0x0f;
  if (c >= 10)
    {
      c = c + 'A' - 10;
    }
  else
    {
      c = c + '0';
    }
  uartSend(c);
}

void uartHexAffiche(unsigned char data)
{
  uartHexAffiche4(data >> 4);
  uartHexAffiche4(data);
}

#endif /* UART_FULL */
